#!/usr/bin/php5

<?php

/**
 * Classes handling the EPP communication with IT-NIC.
 *
 * PHP version 5
 *
 * LICENSE:
 *
 * Copyright (c) 2009, Günther Mair <guenther.mair@hoslo.ch>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1) Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3) Neither the name of Günther Mair nor the names of its contributors may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @category    Net
 * @package     Net_EPP_IT_Client
 * @author      Günther Mair <guenther.mair@hoslo.ch>
 * @license     http://opensource.org/licenses/bsd-license.php New BSD License
 *
 */
 
 /***
 
 Modified by Fabio Ferrero <ferrero@interim.it> on 2012-07-04 (new EPP version)
 Modified by Fabio Ferrero <ferrero@interim.it> on 2010-09-22
 
 One single file stripped down version to only check credit and message queue, warnings are sent by mail, useful for crontab
 
 Depend on: the PEAR package HTTP_Client (handles communication and cookies); install it using "pear install HTTP_Client"
 
 ***/

if ( ! class_exists('HTTP_Client') ) require_once 'HTTP/Client.php';
  
function main() {
  $nic= new Net_EPP_IT_Client("config.xml");
  $session= new Net_EPP_IT_Session($nic);

  if ( ! $session->hello() ) {
    echo "Connection FAILED.\n";
    print_r( $session->result );
  } else {
    if ( $session->login() === FALSE ) {
      echo "Login FAILED (code ".$session->svCode.", '".$session->svMsg."').\n";
    } else {
      $msg= '';
      $credit= sprintf("%.2f", $session->showCredit());
/* commentato perche' la gestione dei messaggi e' fatta da epp_pub.php
      $msgq= $session->pollMessageCount();
      if ($msgq > 0) {
        if ($msgq == 1) {
          $msg .= sprintf("C'e' un messaggio in coda\n");
        } else {
          $msg .= sprintf("Ci sono %s messaggi in coda\n", $msgq);
        }
      }
*/
      if ($credit < $nic->EPPCfg->creditwarning) {
        $msg .= sprintf("ATTENZIONE! Il tuo credito e' di %s euro", $credit);
      }
      if (strlen($msg)) {
        mmail($nic->EPPCfg->mailto, 'EPP NIC warning', $msg, $nic->EPPCfg->mailto);
      }
      if ( ! $session->logout() ) {
        echo "Logout FAILED (code ".$session->svCode.", '".$session->svMsg."').\n";
      }
    }
  }
}

function mmail($to, $subj, $msg, $from) {
  $hdr= sprintf("From: %s\nReturn-Path: %s\nReply-To: %s\n", $from, $from, $from);
  $env= sprintf("-r%s", $from);
  mail($to, $subj, $msg, $hdr, $env);
}

/*** Abstract object ***/

abstract class Net_EPP_IT_AbstractObject {
  protected $client;
  protected $trues = array("true", "TRUE", 1);
  protected $falses = array("false", "FALSE", 0, null);

  public    $xmlQuery;  // xml query string
  public    $result;    // HTTP response string
  public    $xmlResult; // parsed reponse (SimpleXMLElement may be incomplete)

  public    $svCode;
  public    $svMsg;
  public    $extValueReasonCode;
  public    $extValueReason;

  public function __construct(&$client) {
    $this->client= $client;
  }

  protected function error($msg, $code= "0000") {
    $this->svMsg= $msg;
    $this->svCode= $code;
  }

  protected function ExecuteQuery($clTRType, $clTRObject) {
    // send request + parse response
    $this->result= $this->client->sendRequest($this->xmlQuery);
    $this->xmlResult= $this->client->parseResponse($this->result['body']);

    // look for a server response code
    if ( is_object($this->xmlResult->response->result) ) {

      // look for a server message
      if ( is_object($this->xmlResult->response->result->msg) )
        $this->svMsg= $this->xmlResult->response->result->msg;
      else
        $this->svMsg= "";

      // look for a server message code
      $this->svCode= $this->xmlResult->response->result['code'];
      switch ( substr($this->svCode, 0, 1) ) {
        case "1":
          $return_code= TRUE;
          break;
        case "2":
        default:
          $return_code= FALSE;
          break;
      }

      // look for an extended server error message and code
      if ( is_object($this->xmlResult->response->result->extValue->reason) ) {
        $this->extValueReasonCode= $this->xmlResult->response->result->extValue->value->reasonCode;
        $this->extValueReason= $this->xmlResult->response->result->extValue->reason;
      } else {
        $this->extValueReasonCode= '';
        $this->extValueReason= '';
      }

    } else {
      $this->error("Unexpected result (no xml response code).");
      $return_code = FALSE;
    }

    return $return_code;
  }

}

/*** EPP IT Client ***/

class Net_EPP_IT_Client {
  var $EPPCfg;
  var $EPPClient;

  private $DEFAULT_HEADERS= array('content-type' => 'text/xml; charset=UTF-8');

  function __construct($cfg= "config.xml") {
    if ( is_readable($cfg) ) {
      $this->EPPCfg= @simplexml_load_file($cfg);
    } else {
      $this->EPPCfg= @simplexml_load_string($cfg);
      if ( $this->EPPCfg === FALSE ) {
        exit("FATAL ERROR: config file '".$cfg."' not readable or not a XML string\n");
      }
    }
    $this->EPPClient= new HTTP_Client(null, $this->DEFAULT_HEADERS);
  }

  function sendRequest($data) {
    if ( $this->EPPClient->valid() ) $this->EPPClient->next();
    $this->EPPClient->post($this->EPPCfg->server, utf8_encode($data), true);
    return $this->fetchResponse();
  }

  function fetchResponse() {
    return $this->EPPClient->current();
  }

  function parseResponse($xml= null) {
    if ( $xml == null ) {
      $response = $this->fetchResponse();
      return @simplexml_load_string($response[body]);
    } else {
      return @simplexml_load_string($xml);
    }
  }

  function __destruct() {
    if ( is_object($this->EPPClient) ) $this->EPPClient->reset();
  }

}

/*** EPP IT Session ***/

class Net_EPP_IT_Session extends Net_EPP_IT_AbstractObject {
  protected $credit= null;
  protected $messages= null;

  public function hello() {
    $this->xmlQuery= file_get_contents('hello.xml');
    $this->ExecuteQuery("hello", "");
    if ( (substr($this->result['code'], 0, 1) == "2") && (is_object($this->xmlResult->greeting)) ) {
      return TRUE;
    } else {
      return FALSE;
    }
  }

  private function loginout($which) {
    if ( $this->ExecuteQuery($which, "") ) {
      if ( is_object($this->xmlResult->response->extension) ) {
        $tmp= $this->xmlResult->response->extension->children('http://www.nic.it/ITNIC-EPP/extepp-2.0');
        $this->credit= (float) $tmp->creditMsgData->credit;
      }
      return TRUE;
    } else {
      return FALSE;
    }
  }

  public function login() {
    $xml= file_get_contents('login.xml');
    $xml= str_replace('$$REG-NAME$$', $this->client->EPPCfg->username, $xml);
    $xml= str_replace('$$REG-PWD$$', $this->client->EPPCfg->password, $xml);
    $this->xmlQuery= $xml;
    return $this->loginout('login');
  }

  public function keepalive() {
    return $this->hello();
  }

  public function logout() {
    $xml= file_get_contents('logout.xml');
    $this->xmlQuery= $xml;
    return $this->loginout('logout');
  }

  public function pollMessageCount() {
    $this->poll();
    return (int)$this->messages;
  }

  public function poll() {
    $this->xmlQuery= file_get_contents("poll.xml");
    $qrs= $this->ExecuteQuery("poll", "poll");
    if ( is_object($this->xmlResult->response->msgQ[0]) ) {
      $this->messages= (int)$this->xmlResult->response->msgQ->attributes()->count;
    } else if ( $qrs === TRUE ) {
      $this->messages= 0;
    }
    return $qrs;
  }

  public function showCredit() {
    return $this->credit;
  }
}

main();

?>
